package org.jfree.chart.renderer.xy;

import java.awt.Graphics2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.util.Iterator;
import java.util.List;

import org.jfree.chart.axis.AxisState;
import org.jfree.chart.axis.NumberTick;
import org.jfree.chart.axis.TickType;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.XYPolarPlot;
import org.jfree.text.TextUtilities;
import org.jfree.ui.RectangleEdge;
import org.jfree.ui.TextAnchor;

/**
 * A renderer that can be used with the {@link XYPolarPlot} class.
 */
public class DefaultXYPolarItemRenderer extends DefaultXYItemRenderer {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/**
	 * Draw the angular gridlines - the spokes.
	 * 
	 * @param g2
	 *            the drawing surface.
	 * @param plot
	 *            the plot.
	 * @param ticks
	 *            the ticks.
	 * @param dataArea
	 *            the data area.
	 */
	public void drawAngularGridLines(Graphics2D g2, XYPolarPlot plot,
			List ticks, Rectangle2D dataArea) {
		// find outer-radius
		double r = 2.0 * Math.max(plot.getMaxRadius(), plot.getMaxUsedRadius());
		double rUsed = plot.getMaxUsedRadius();

		AxisState state = new AxisState();
		List list = plot.getRangeAxis().refreshTicks(g2, state, dataArea,
				RectangleEdge.LEFT);
		for (int k = list.size() - 1; k >= 0; k--) {
			NumberTick t = (NumberTick) list.get(k);
			if (t.getTickType().equals(TickType.MAJOR) && (t.getValue() <= r)
					&& (rUsed <= t.getValue())) {
				r = t.getValue();
			}

		}

		ValueAxis axis = plot.getDomainAxis();

		g2.setFont(axis.getTickLabelFont());
		g2.setStroke(plot.getRangeGridlineStroke());
		g2.setPaint(plot.getRangeGridlinePaint());

		double transX1 = plot.getDomainAxis().valueToJava2D(0.0, dataArea,
				plot.getDomainAxisEdge());
		double transY1 = plot.getRangeAxis().valueToJava2D(0.0, dataArea,
				plot.getRangeAxisEdge());

		// for all ticks
		Iterator iterator = ticks.iterator();
		while (iterator.hasNext()) {
			NumberTick tick = (NumberTick) iterator.next();
			double transX2 = plot.getDomainAxis().valueToJava2D(
					r * Math.cos((tick.getValue() * Math.PI) / 180.0),
					dataArea, plot.getDomainAxisEdge());
			double transY2 = plot.getRangeAxis().valueToJava2D(
					r * Math.sin((tick.getValue() * Math.PI) / 180.0),
					dataArea, plot.getRangeAxisEdge());

			// minor grid and tick
			if (tick.getTickType().equals(TickType.MINOR)) {
				if (plot.isDomainMinorGridlinesVisible()) {
					g2.setStroke(plot.getDomainMinorGridlineStroke());
					g2.setPaint(plot.getDomainMinorGridlinePaint());
					g2.drawLine((int) transX1, (int) transY1, (int) transX2,
							(int) transY2);
				}
				if (axis.isMinorTickMarksVisible()) {
					double transX3 = plot
							.getDomainAxis()
							.valueToJava2D(
									r
											* 1.005
											* Math.cos((tick.getValue() * Math.PI) / 180.0),
									dataArea, plot.getDomainAxisEdge());
					double transY3 = plot
							.getRangeAxis()
							.valueToJava2D(
									r
											* 1.005
											* Math.sin((tick.getValue() * Math.PI) / 180.0),
									dataArea, plot.getRangeAxisEdge());

					g2.setStroke(axis.getTickMarkStroke());
					g2.setPaint(axis.getTickMarkPaint());
					g2.drawLine((int) transX2, (int) transY2, (int) transX3,
							(int) transY3);
				}

			}

			// major grid, tick and tick-label
			if (tick.getTickType().equals(TickType.MAJOR)) {
				if (plot.isDomainGridlinesVisible()) {
					g2.setStroke(plot.getDomainGridlineStroke());
					g2.setPaint(plot.getDomainGridlinePaint());
					g2.drawLine((int) transX1, (int) transY1, (int) transX2,
							(int) transY2);
				}
				if (axis.isTickMarksVisible()) {
					double transX3 = plot
							.getDomainAxis()
							.valueToJava2D(
									r
											* 1.01
											* Math.cos((tick.getValue() * Math.PI) / 180.0),
									dataArea, plot.getDomainAxisEdge());
					double transY3 = plot
							.getRangeAxis()
							.valueToJava2D(
									r
											* 1.01
											* Math.sin((tick.getValue() * Math.PI) / 180.0),
									dataArea, plot.getRangeAxisEdge());

					g2.setStroke(axis.getTickMarkStroke());
					g2.setPaint(axis.getTickMarkPaint());
					g2.drawLine((int) transX2, (int) transY2, (int) transX3,
							(int) transY3);
				}
				if (axis.isTickLabelsVisible()) {
					double transX3 = plot
							.getDomainAxis()
							.valueToJava2D(
									r
											* 1.05
											* Math.cos((tick.getValue() * Math.PI) / 180.0),
									dataArea, plot.getDomainAxisEdge());
					double transY3 = plot
							.getRangeAxis()
							.valueToJava2D(
									r
											* 1.05
											* Math.sin((tick.getValue() * Math.PI) / 180.0),
									dataArea, plot.getRangeAxisEdge());
					g2.setPaint(axis.getLabelPaint());
					TextUtilities
							.drawAlignedString(tick.getText(), g2,
									(float) transX3, (float) transY3,
									TextAnchor.CENTER);
				}
			}

		}

		// axis-line
		if (axis.isAxisLineVisible()) {
			double transX2 = plot.getDomainAxis().valueToJava2D(r, dataArea,
					plot.getDomainAxisEdge());
			double transY2 = plot.getRangeAxis().valueToJava2D(r, dataArea,
					plot.getRangeAxisEdge());
			g2.setStroke(axis.getAxisLineStroke());
			g2.setPaint(axis.getAxisLinePaint());
			Ellipse2D ring = new Ellipse2D.Double(
					transX1 - (transX2 - transX1), transY1
							- (transY1 - transY2), 2 * (transX2 - transX1),
					2 * (transY1 - transY2));
			g2.draw(ring);
		}

	}

	/**
	 * Draw the radial gridlines - the rings.
	 * 
	 * @param g2
	 *            the drawing surface.
	 * @param plot
	 *            the plot.
	 * @param radialAxis
	 *            the radial axis.
	 * @param ticks
	 *            the ticks.
	 * @param dataArea
	 *            the data area.
	 */
	public void drawRadialGridLines(Graphics2D g2, XYPolarPlot plot,
			ValueAxis radialAxis, List ticks, Rectangle2D dataArea) {
		// find outer-radius
		double r = 2.0 * Math.max(plot.getMaxRadius(), plot.getMaxUsedRadius());
		double rUsed = plot.getMaxUsedRadius();

		AxisState state = new AxisState();
		List list = plot.getRangeAxis().refreshTicks(g2, state, dataArea,
				RectangleEdge.LEFT);
		for (int k = list.size() - 1; k >= 0; k--) {
			NumberTick t = (NumberTick) list.get(k);
			if (t.getTickType().equals(TickType.MAJOR) && (t.getValue() <= r)
					&& (rUsed <= t.getValue())) {
				r = t.getValue();
			}

		}

		ValueAxis axis = plot.getRangeAxis();

		g2.setFont(plot.getDomainAxis().getTickLabelFont());
		g2.setStroke(plot.getRangeGridlineStroke());
		g2.setPaint(plot.getRangeGridlinePaint());

		double transX1 = plot.getDomainAxis().valueToJava2D(0.0, dataArea,
				plot.getDomainAxisEdge());
		double transY1 = plot.getRangeAxis().valueToJava2D(0.0, dataArea,
				plot.getRangeAxisEdge());

		// for all ticks
		Iterator iterator = ticks.iterator();
		while (iterator.hasNext()) {
			NumberTick tick = (NumberTick) iterator.next();
			double val = tick.getValue();
			if ((0.0 < val) && (val <= r)) {
				double transX2 = plot.getDomainAxis().valueToJava2D(val,
						dataArea, plot.getDomainAxisEdge());
				double transY2 = plot.getRangeAxis().valueToJava2D(val,
						dataArea, plot.getRangeAxisEdge());

				// minor grid and tick
				if (tick.getTickType().equals(TickType.MINOR)) {
					if (plot.isRangeMinorGridlinesVisible()) {
						g2.setStroke(plot.getRangeMinorGridlineStroke());
						g2.setPaint(plot.getRangeMinorGridlinePaint());

						Ellipse2D ring = new Ellipse2D.Double(transX1
								- (transX2 - transX1), transY1
								- (transY1 - transY2), 2 * (transX2 - transX1),
								2 * (transY1 - transY2));
						g2.draw(ring);
					}
					if (axis.isMinorTickMarksVisible()) {
						double transX3 = plot.getDomainAxis().valueToJava2D(
								r * 0.005, dataArea, plot.getDomainAxisEdge());
						double transY3 = plot.getRangeAxis().valueToJava2D(val,
								dataArea, plot.getRangeAxisEdge());
						g2.setStroke(axis.getTickMarkStroke());
						g2.setPaint(axis.getTickMarkPaint());
						g2.drawLine((int) transX1, (int) transY2,
								(int) transX3, (int) transY3);

					}
				}

				// major grid, tick and tick-label
				if (tick.getTickType().equals(TickType.MAJOR)) {
					if (plot.isRangeGridlinesVisible()) {
						g2.setStroke(plot.getRangeGridlineStroke());
						g2.setPaint(plot.getRangeGridlinePaint());

						Ellipse2D ring = new Ellipse2D.Double(transX1
								- (transX2 - transX1), transY1
								- (transY1 - transY2), 2 * (transX2 - transX1),
								2 * (transY1 - transY2));
						g2.draw(ring);
					}
					if (axis.isTickMarksVisible()) {
						double transX3 = plot.getDomainAxis().valueToJava2D(
								r * 0.01, dataArea, plot.getDomainAxisEdge());
						double transY3 = plot.getRangeAxis().valueToJava2D(val,
								dataArea, plot.getRangeAxisEdge());
						g2.setStroke(axis.getTickMarkStroke());
						g2.setPaint(axis.getTickMarkPaint());
						g2.drawLine((int) transX1, (int) transY2,
								(int) transX3, (int) transY3);
					}
					if (axis.isTickLabelsVisible()) {
						double transX3 = plot.getDomainAxis().valueToJava2D(
								0.00, dataArea, plot.getDomainAxisEdge());
						double transY3 = plot.getRangeAxis().valueToJava2D(val,
								dataArea, plot.getRangeAxisEdge());
						g2.setPaint(axis.getLabelPaint());
						TextUtilities.drawAlignedString(tick.getText(), g2,
								(float) transX3, (float) transY3,
								TextAnchor.CENTER);
					}
				}

			}
		}

		// axis-line
		if (axis.isAxisLineVisible()) {
			double transY2 = plot.getRangeAxis().valueToJava2D(r, dataArea,
					plot.getRangeAxisEdge());
			g2.setStroke(axis.getAxisLineStroke());
			g2.setPaint(axis.getAxisLinePaint());
			g2.drawLine((int) transX1, (int) transY1, (int) transX1,
					(int) transY2);
		}
	}
}