/*
 * $LastChangedRevision: 864 $
 * $LastChangedDate:: 2016-02-04 14:25:17 +0100#$
 *
 * Copyright (c) 2013, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
#include "Scope_Main.h"

/* private prototypes */
static uint8 isTriggerEvent(SCOPE_MAIN *pTScope, int32 curTrgValue);
static int32 getTriggerValue(SCOPE_MAIN *pTScope);
static void sampleData(SCOPE_MAIN *pTScope);

ALIGNTYPE ScopeArray[SCOPE_SIZE];

#if defined(__COMPILER_CODEWARRIOR_ECLIPSE__)
#pragma push
#pragma section sdata_type ".ptrTableStruct" ".ptrTableStruct"
extern volatile tTableStruct *TableStruct;
#pragma pop
#else
extern volatile tTableStruct* TableStruct;
#endif

/* private prototypes */
static void* getBlockAddress(const tParameterTable* paramTable, uint16 paramId);
static tBlockFunctions* getBlockFunction(const tBlockFunctions* blockFuncTable, uint16 blockId);

#define SOURCE_TYPE_ADDRESS ((uint8)0)
#define SOURCE_TYPE_CONTROLBLOCK ((uint8)1)
#define SOURCE_TYPE_INPORTBLOCK ((uint8)2)
#define SOURCE_TYPE_OUTPORTBLOCk ((uint8)3)

#define TRG_MODE_AUTO ((uint8)0)
#define TRG_MODE_NORMAL ((uint8)1)

/************************************************/
/*  Scope_Main_Init                             */
/************************************************/
void Scope_Main_Init(SCOPE_MAIN *pTScope)
{
	uint8 i;

	pTScope->ID = SCOPE_MAIN_ID;
	for (i=0;i<MAX_SCOPE_CHANNELS;i++)
	{
		pTScope->channelAddr[i] = (void*)0;
		pTScope->dataSize[i] = (uint8)0;
	}

	pTScope->arrayAddr = (void*)ScopeArray;
	pTScope->trgLevel = (int32)0;
	pTScope->trgLastValue = (int32)0;
	pTScope->trgAddr = (void*)0;
	pTScope->trgDataType = (uint8)0;
	pTScope->state = SCOPE_IDLE;
	pTScope->offlinePtr = (uint32)0;
	pTScope->dataSizeTotal = (uint8)0;
	pTScope->noChannels = (uint8)0;
	pTScope->stf = (uint16)0;
    pTScope->stfCnt = (uint16)0;
	pTScope->timestamp = (uint16)0;
    pTScope->txFrameSize = (uint8)0;
	pTScope->trgDelay = (int32)0;
	pTScope->trgDelayCnt = (int32)0;
	pTScope->trgCountReached = (uint8)0;
	pTScope->maxUsedLength = SCOPE_SIZE;
	pTScope->trgEdge = EDGE_RISING;
	pTScope->arraySize = SCOPE_SIZE;

	TableStruct->piScope = pTScope;
}



/************************************************/
/*  Scope_Main_Update                           */
/************************************************/
void Scope_Main_Update(SCOPE_MAIN *pTScope)
{
    uint8 i, j;
	int32 curTrgValue;

	switch (pTScope->state)
	{
		case SCOPE_IDLE:
			break;
		case SCOPE_SAMPLE_ONLINE:
			pTScope->timestamp++;
            pTScope->stfCnt++;
            break;
        case SCOPE_SAMPLE_OFFLINE:
            /* check if sample time prescaler is reached */
            if (pTScope->stfCnt++ >= pTScope->stf)
            {
                pTScope->stfCnt = (uint16)0;

				sampleData(pTScope);

                /* if size of next data size would exceed SCOPE_SIZE, the */
                /* offline sampling mode will be stopped (change to idle state) */
                if ((pTScope->offlinePtr + \
                    pTScope->dataSizeTotal) > SCOPE_SIZE)
                {
                    pTScope->state = SCOPE_IDLE;
                }
            }
            pTScope->timestamp++;
            break;
        /* wait for trigger on rising edge (online mode) */
        case SCOPE_WAITTRGPOS_ONLINE:
            switch (pTScope->trgDataType)
            {
                /* 8 bit */
                case 1:
                    if (((int8)pTScope->trgLastValue < (int8)pTScope->trgLevel) && \
                        (*(int8*)pTScope->trgAddr >= (int8)pTScope->trgLevel))
			        {
				        pTScope->state = SCOPE_SAMPLE_ONLINE;
			        }
			        else
			        {
				        pTScope->trgLastValue = *(int8*)pTScope->trgAddr;
			        }
			        break;
                /* 16 bit */
                case 2:
                    if (((int16)pTScope->trgLastValue < (int16)pTScope->trgLevel) && \
                        (*(int16*)pTScope->trgAddr >= (int16)pTScope->trgLevel))
			        {
				        pTScope->state = SCOPE_SAMPLE_ONLINE;
			        }
			        else
			        {
				        pTScope->trgLastValue = *(int16*)pTScope->trgAddr;
			        }
			        break;
                /* 32 bit */
                case 4:
                    if (((int32)pTScope->trgLastValue < (int32)pTScope->trgLevel) && \
                        (*(int32*)pTScope->trgAddr >= (int32)pTScope->trgLevel))
			        {
				        pTScope->state = SCOPE_SAMPLE_ONLINE;
			        }
			        else
			        {
				        pTScope->trgLastValue = *(int32*)pTScope->trgAddr;
			        }
			        break;
            }
            break;
        /* wait for trigger on falling edge (online mode) */
        case SCOPE_WAITTRGNEG_ONLINE:
            switch (pTScope->trgDataType)
            {
                /* 8 bit */
                case 1:
                    if (((int8)pTScope->trgLastValue > (int8)pTScope->trgLevel) && \
                        (*(int8*)pTScope->trgAddr <= (int8)pTScope->trgLevel))
			        {
				        pTScope->state = SCOPE_SAMPLE_ONLINE;
			        }
			        else
			        {
				        pTScope->trgLastValue = *(int8*)pTScope->trgAddr;
			        }
			        break;
                /* 16 bit */
                case 2:
                    if (((int16)pTScope->trgLastValue > (int16)pTScope->trgLevel) && \
                        (*(int16*)pTScope->trgAddr <= (int16)pTScope->trgLevel))
			        {
				        pTScope->state = SCOPE_SAMPLE_ONLINE;
			        }
			        else
			        {
				        pTScope->trgLastValue = *(int16*)pTScope->trgAddr;
			        }
			        break;
                /* 32 bit */
                case 4:
                    if (((int32)pTScope->trgLastValue > (int32)pTScope->trgLevel) && \
                        (*(int32*)pTScope->trgAddr <= (int32)pTScope->trgLevel))
			        {
				        pTScope->state = SCOPE_SAMPLE_ONLINE;
			        }
			        else
			        {
				        pTScope->trgLastValue = *(int32*)pTScope->trgAddr;
			        }
			        break;
            }
            break;

        /* wait for trigger on rising/falling edge (offline mode) */
		case SCOPE_WAITTRG_OFFLINE:
			/* get actual trigger source as 32 bit value */
			curTrgValue = getTriggerValue(pTScope);

			if (pTScope->stfCnt++ >= pTScope->stf)
			{
				pTScope->stfCnt = (uint16)0;
				if (pTScope->trgDelayCnt + pTScope->dataSizeTotal > SCOPE_SIZE)
				{
					pTScope->trgDelayCnt = (int32)0;
				}

				i = 0;
	            do
	               {
	                j = 0;
	                do
	                {
	                   	*(ALIGNCASTPTR pTScope->arrayAddr + pTScope->trgDelayCnt++) = \
	                   		*(ALIGNCASTPTR pTScope->channelAddr[i] + j);
	                    j++;
	                }
	                while (j < pTScope->dataSize[i]);
	                i++;
	            }
	            while(i < pTScope->noChannels);

				if (pTScope->trgDelayCnt >= pTScope->trgDelay)
				{
					pTScope->trgCountReached = 1;
				}
			}

			if (isTriggerEvent(pTScope, curTrgValue) && pTScope->trgCountReached)
			{
				if (pTScope->trgDelay <= (int32)0)
				{
					pTScope->trgDelayCnt = pTScope->trgDelay;
					pTScope->state = SCOPE_WAIT_TRG_NEG_DELAY;
				}
				else
				{
					pTScope->offlinePtr = (uint32)pTScope->trgDelayCnt;
					pTScope->state = SCOPE_TRG_SAMPLE_OFFLINE;
				}
			}
			else
			{
				pTScope->trgLastValue = curTrgValue;
			}
			break;

		case SCOPE_WAIT_TRG_NEG_DELAY:
			/* in case of negative delay wait until delay time has been passed */
			if (pTScope->stfCnt++ >= pTScope->stf)
			{
				pTScope->stfCnt = (uint16)0;

				pTScope->trgDelayCnt += pTScope->dataSizeTotal;
				if (pTScope->trgDelayCnt >= 0)
				{
					pTScope->state = SCOPE_SAMPLE_OFFLINE;
				}
			}
			break;

		case SCOPE_TRG_SAMPLE_OFFLINE:
            if (pTScope->stfCnt++ >= pTScope->stf)
            {
                pTScope->stfCnt = (uint16)0;

				if ((pTScope->offlinePtr + pTScope->dataSizeTotal) > SCOPE_SIZE)
				{
					pTScope->offlinePtr = (uint32)0;
				}

				sampleData(pTScope);

				if (((pTScope->trgDelayCnt - pTScope->offlinePtr) == pTScope->trgDelay) || \
					((pTScope->trgDelayCnt - pTScope->offlinePtr + pTScope->maxUsedLength) <= pTScope->trgDelay))
				{
					pTScope->state = SCOPE_IDLE;
				}

			}
			break;
	}
}

/************************************************/
/*  Scope_Main_Load                             */
/************************************************/
uint8 Scope_Main_Load(SCOPE_MAIN *pTScope,uint8 *ucData)
{
   ucData[0] = pTScope->state;
   ucData[1] = pTScope->noChannels;
   ucData[2] = (uint8)(pTScope->stf & 0xFF);
   ucData[3] = (uint8)((pTScope->stf >> 8) & 0xFF);
   ucData[4] = (uint8)(pTScope->offlinePtr & 0xFF);
   ucData[5] = (uint8)((pTScope->offlinePtr >> 8) & 0xFF);
   ucData[6] = (uint8)((pTScope->offlinePtr >> 16) & 0xFF);
   ucData[7] = (uint8)((pTScope->offlinePtr >> 24) & 0xFF);
   ucData[8] = (uint8)((uint32)pTScope->arrayAddr & 0xFF);
   ucData[9] = (uint8)(((uint32)pTScope->arrayAddr >> 8) & 0xFF);
   ucData[10] = (uint8)(((uint32)pTScope->arrayAddr >> 16) & 0xFF);
   ucData[11] = (uint8)(((uint32)pTScope->arrayAddr >> 24) & 0xFF);

   ucData[12] = (uint8)(pTScope->trgDelay & 0xFF);
   ucData[13] = (uint8)((pTScope->trgDelay >> 8) & 0xFF);
   ucData[14] = (uint8)((pTScope->trgDelay >> 16) & 0xFF);
   ucData[15] = (uint8)((pTScope->trgDelay >> 24) & 0xFF);
   ucData[16] = (uint8)(pTScope->trgDelayCnt & 0xFF);
   ucData[17] = (uint8)((pTScope->trgDelayCnt >> 8) & 0xFF);
   ucData[18] = (uint8)((pTScope->trgDelayCnt >> 16) & 0xFF);
   ucData[19] = (uint8)((pTScope->trgDelayCnt >> 24) & 0xFF);

   ucData[20] = (uint8)(pTScope->maxUsedLength & 0xFF);
   ucData[21] = (uint8)((pTScope->maxUsedLength >> 8) & 0xFF);
   ucData[22] = (uint8)((pTScope->maxUsedLength >> 16) & 0xFF);
   ucData[23] = (uint8)((pTScope->maxUsedLength >> 24) & 0xFF);

   ucData[24] = (uint8)(pTScope->arraySize & 0xFF);
   ucData[25] = (uint8)((pTScope->arraySize >> 8) & 0xFF);
   ucData[26] = (uint8)((pTScope->arraySize >> 16) & 0xFF);
   ucData[27] = (uint8)((pTScope->arraySize >> 24) & 0xFF);

   return ((uint8)28);
}

/************************************************/
/*  Scope_Main_Save                             */
/************************************************/
uint8 Scope_Main_Save(SCOPE_MAIN *pTScope, uint8 *ucData, uint8 ucFRMlen)
{
   const uint8 offset = (uint8)4;
   uint8 i = (uint8)0;
   uint8 ptr = offset;

   uint8 sourceType;
   uint16 paramId, elementId, blockId;
   void* blockAddr;
   tBlockFunctions* blockFunctions;

   if (ucFRMlen < offset)
       return ((uint8)1);
   else
   {
       /* send error if selected channels > max channels */
       if (ucData[1] > MAX_SCOPE_CHANNELS)
       {
    	   return ((uint8)1);
       }
       /* send error if no channels are configured */
       else if (ucData[1] == (uint8)0)
       {
    	   return ((uint8)1);
       }

	   /* IMPORTANT - DO NOT MOVE THIS SETION TO OTHER PARTS */
	   /* OF THE SAVE FUNCTION */
	   /* set state to idle & number of inputs to 0 to avoid */
	   /* undefined behaviour while updating the addresses */
	   /* & scope values if Scope Update function is */
	   /* executed in sample time irq */
	   pTScope->state = SCOPE_IDLE;
	   pTScope->noChannels = (uint8)0;

       /* Scope Save Frame contains the following parameters: */
       /* - Scope State (1 byte) */
       /* - Number of Inputs (1 byte) */
       /* - Sample Time Prescaler (2 bytes) */
	   /* - Channel #1 Source type (1 byte) */
       /* - Channel #1 Address (4 bytes) */
       /* - Channel #1 DataType (1 byte) */
       /* - ... */
	   /* - Channel #n Source type (1 byte) */
       /* - Channel #n Address (2 bytes) */
       /* - Channel #n DataType (1 byte) */
       /* - Trigger DataType (1 byte) */
	   /* - Trigger Source type (1 byte) */
       /* - Trigger Address (4 bytes) */
       /* - Trigger Level (4 bytes) */
       pTScope->timestamp = (uint16)0;
       pTScope->offlinePtr = (uint32)0;
       pTScope->stf = (uint16)ucData[2] + ((uint16)ucData[3] << 8);
       pTScope->stfCnt = pTScope->stf;
       pTScope->trgLastValue = (int32)0;

       pTScope->dataSizeTotal = (uint8)0;
       /* 2 bytes for 16 bit timestamp */
       pTScope->txFrameSize = (uint8)2;

	   pTScope->trgCountReached = (uint8)0;
	   pTScope->trgDelayCnt = (int32)0;

       while (i < ucData[1])
       {
    	   sourceType = ucData[ptr];
    	   ptr++;

    	   switch (sourceType)
    	   {
    	   case SOURCE_TYPE_CONTROLBLOCK:
			   elementId = (uint16)ucData[ptr] + ((uint16)ucData[ptr + 1] << 8);
			   paramId = (uint16)ucData[ptr + 2] + ((uint16)ucData[ptr + 3] << 8);

			   blockAddr = getBlockAddress(TableStruct->TParamTable, paramId);
			   if (blockAddr == (void*)0)
			   {
				   return ((uint8)1);
			   }
			   blockId = *(uint16*)blockAddr;
			   blockFunctions = getBlockFunction(TableStruct->TFncTable, blockId);
			   if (blockFunctions == (tBlockFunctions*)0)
			   {
				   return ((uint8)1);
			   }
			   pTScope->channelAddr[i] = blockFunctions->pFGetAddress(blockAddr, elementId);
			   if (pTScope->channelAddr[i] == (void*)0)
			   {
				   return ((uint8)1);
			   }
			   break;
    	   case SOURCE_TYPE_INPORTBLOCK:
    		   paramId = (uint16)ucData[ptr + 2] + ((uint16)ucData[ptr + 3] << 8);
			   blockAddr = getBlockAddress(TableStruct->TParamTable, paramId);
			   if (blockAddr == (void*)0)
			   {
				   return ((uint8)1);
			   }
			   pTScope->channelAddr[i] = blockAddr;
    		   break;
    	   case SOURCE_TYPE_OUTPORTBLOCk:
    		   paramId = (uint16)ucData[ptr + 2] + ((uint16)ucData[ptr + 3] << 8);
			   blockAddr = getBlockAddress(TableStruct->TParamTable, paramId);
			   if (blockAddr == (void*)0)
			   {
				   return ((uint8)1);
			   }
			   pTScope->channelAddr[i] = *(void**)blockAddr;
    		   break;
    	   case SOURCE_TYPE_ADDRESS:
    		   pTScope->channelAddr[i] = (void*)((uint32)ucData[ptr] + ((uint32)ucData[ptr + 1] << 8) + \
    				   ((uint32)ucData[ptr + 2] << 16) + ((uint32)ucData[ptr + 3] << 24));
    		   break;
    	   default:
    		   return ((uint8)1);
    	   }

          pTScope->dataSize[i] = (uint16)ucData[ptr + 4];

          pTScope->dataSizeTotal += pTScope->dataSize[i];
#if defined(__DATA_WIDTH_16BIT__)
          pTScope->txFrameSize += (pTScope->dataSize[i]*2);
#elif defined(__DATA_WIDTH_8BIT__)
          pTScope->txFrameSize += (pTScope->dataSize[i]);
#else
#error DATA WIDTH NOT DEFINED
#endif

          pTScope->maxUsedLength = SCOPE_SIZE - \
        		  (SCOPE_SIZE % pTScope->dataSizeTotal);

          ptr += (uint8)5;
          i++;
       }

       /* trigger parameters */
       pTScope->trgDataType = ucData[ptr];
       ptr++;
       sourceType = ucData[ptr];
       ptr++;
       switch (sourceType)
       {
	   case SOURCE_TYPE_CONTROLBLOCK:
		   elementId = (uint16)ucData[ptr] + ((uint16)ucData[ptr + 1] << 8);
		   paramId = (uint16)ucData[ptr + 2] + ((uint16)ucData[ptr + 3] << 8);

		   blockAddr = getBlockAddress(TableStruct->TParamTable, paramId);
		   if (blockAddr == (void*)0)
		   {
			   return ((uint8)1);
		   }
		   blockId = *(uint16*)blockAddr;
		   blockFunctions = getBlockFunction(TableStruct->TFncTable, blockId);
		   if (blockFunctions == (tBlockFunctions*)0)
		   {
			   return ((uint8)1);
		   }
		   pTScope->trgAddr = blockFunctions->pFGetAddress(blockAddr, elementId);
		   if (pTScope->trgAddr == (void*)0)
		   {
			   return ((uint8)1);
		   }
		   break;
	   case SOURCE_TYPE_INPORTBLOCK:
		   paramId = (uint16)ucData[ptr + 2] + ((uint16)ucData[ptr + 3] << 8);
		   blockAddr = getBlockAddress(TableStruct->TParamTable, paramId);
		   if (blockAddr == (void*)0)
		   {
			   return ((uint8)1);
		   }
		   pTScope->trgAddr = blockAddr;
		   break;
	   case SOURCE_TYPE_OUTPORTBLOCk:
		   paramId = (uint16)ucData[ptr + 2] + ((uint16)ucData[ptr + 3] << 8);
		   blockAddr = getBlockAddress(TableStruct->TParamTable, paramId);
		   if (blockAddr == (void*)0)
		   {
			   return ((uint8)1);
		   }
		   pTScope->trgAddr = *(void**)blockAddr;
		   break;
	   case SOURCE_TYPE_ADDRESS:
		   pTScope->trgAddr = (void*)((uint32)ucData[ptr] + ((uint32)ucData[ptr + 1] << 8) + \
				   ((uint32)ucData[ptr + 2] << 16) + ((uint32)ucData[ptr + 3] << 24));
		   break;
	   default:
		   return ((uint8)1);
       }
       ptr += (uint8)4;
       pTScope->trgLevel = ((int32)ucData[ptr] + \
          ((int32)ucData[ptr + 1] << 8) + \
          ((int32)ucData[ptr + 2] << 16) + \
          ((int32)ucData[ptr + 3] << 24));

	   ptr += (uint8)4;
	   pTScope->trgDelay = ((int32)ucData[ptr] + \
			   ((int32)ucData[ptr + 1] << 8) + \
			   ((int32)ucData[ptr + 2] << 16) + \
			   ((int32)ucData[ptr + 3] << 24));

	   ptr += (uint8)4;
	   pTScope->trgEdge = (tTrgEdgeType)ucData[ptr];
       
       ptr++;
       /* check if trigger mode information is being sent (compatibility check for previous Scope versions) */
       if (ucFRMlen > ptr)
       {
           /* get current trigger value if trigger mode is set to NORMAL */
           if (ucData[ptr] == TRG_MODE_NORMAL)
           {
               /* current value @ trg addr = last trg value */
               /* (avoids immediate trg event due to default last */
               /* trg value = 0 and current trg value > trg level */
               pTScope->trgLastValue = getTriggerValue(pTScope);
           }
       }
       else
       {
           pTScope->trgLastValue = getTriggerValue(pTScope);
       }

	   /* IMPORTANT - DO NOT MOVE THIS SETION TO OTHER PARTS */
	   /* OF THE SAVE FUNCTION */
	   /* 'number of inputs' & 'state' moved to end of save function */
	   /* to avoid undefined behaviour if save function is */
	   /* interrupted by sample time irq & program is executed */
	   /* in sample time irq */
	   pTScope->noChannels = ucData[1];
	   pTScope->state = (tScopeState)ucData[0];

       return ((uint8)0);
   }
}

/**
 * Returns block element address.
 */
void* Scope_Main_GetAddress(const SCOPE_MAIN* block, uint16 elementId)
{
	/* Scope doesn't have any elements so return null pointer */
	void* addr = (void*)0;
	return (addr);
}


/* this function will return 1, if the trigger was hit and */
/* 0, if the trigger was not hit */
static uint8 isTriggerEvent(SCOPE_MAIN *pTScope, int32 curTrgValue)
{
	/* rising edge */
	if (pTScope->trgEdge == EDGE_RISING)
	{
		if ((pTScope->trgLastValue < pTScope->trgLevel) && \
			(curTrgValue >= pTScope->trgLevel))
		{
			return (1);
		}
		else
		{
			return (0);
		}
	}
	/* falling edge */
	else
	{
		if ((pTScope->trgLastValue > pTScope->trgLevel) && \
			(curTrgValue <= pTScope->trgLevel))
		{
			return (1);
		}
		else
		{
			return (0);
		}
	}
}

/* return current trigger value as int32 value */
static int32 getTriggerValue(SCOPE_MAIN *pTScope)
{
   int32 lLastTrgValue;
   switch (pTScope->trgDataType)
   {
      /* 8 bit */
      case 1:
		 lLastTrgValue = (*(int8*)pTScope->trgAddr);
		 lLastTrgValue <<= 24;
		 lLastTrgValue >>= 24;
         break;
      /* 16 bit */
      case 2:
		 lLastTrgValue = (int32)(*(int16*)pTScope->trgAddr);
         break;
      /* 32 bit */
      case 4:
		 lLastTrgValue = (*(int32*)pTScope->trgAddr);
         break;
      /* invalid type identifier */
      default:
         /* do nothing */
		 break;
   }
   return (lLastTrgValue);
}

/* sample offline data and write it into offline array */
static void sampleData(SCOPE_MAIN *pTScope)
{
	uint8 i,j;

	i = 0;
    do
    {
    	j = 0;
        do
        {
        	*(ALIGNCASTPTR pTScope->arrayAddr + pTScope->offlinePtr++) = \
				*(ALIGNCASTPTR pTScope->channelAddr[i] + j);
            j++;
        }
        while (j < pTScope->dataSize[i]);

        i++;
    }
    while(i < pTScope->noChannels);
}

/*
 * Returns block functions from block function table.
 */
static tBlockFunctions* getBlockFunction(const tBlockFunctions* blockFuncTable, uint16 blockId)
{
	uint16 i = 0;
	tBlockFunctions* blockFunctionAddr;

	while ((blockId != blockFuncTable[i].iBlockID) && (blockFuncTable[i].iBlockID != 0))
	{
		i++;
	}
	/* check if Block ID can be found in function table */
	if (blockFuncTable[i].iBlockID == 0)
	{
		blockFunctionAddr = (tBlockFunctions*)0;
	}
	else
	{
		blockFunctionAddr = (tBlockFunctions*)&blockFuncTable[i];
	}
	return (blockFunctionAddr);
}

/*
 * Returns block address from parameter table.
 */
static void* getBlockAddress(const tParameterTable* paramTable, uint16 paramId)
{
	void* blockAddr;
	uint16 i;

	/* send parameter ID error if no parameter table has been initialized */
	if (paramTable == 0)
	{
		blockAddr = (void*)0;
	}
	else
	{
		i = 0;
		while((paramId != paramTable[i].uiParID) && (paramTable[i].uiParID != 0))
		{
			i++;
		}
		/* check if parameter is located in parameter table */
		if (paramTable[i].uiParID == 0)
		{
			blockAddr = (void*)0;
		}
		else
		{
			blockAddr = paramTable[i].pAdr;
		}
	}
	return (blockAddr);
}
